
#include <linux/types.h>
#include <linux/version.h>
#include <linux/init.h>
#include <linux/platform_device.h>
#include <linux/clkdev.h>

#include <asm/setup.h>
#include <asm/system_info.h>
#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <asm/mach/time.h>
#include <asm/clkdev.h>

#include <mach/clkdev.h>
#include <mach/hardware.h>
#include <mach/memory.h>
#include <mach/io_map.h>

#include <plat/bsp.h>
#include <plat/mpcore.h>
#include <plat/plat-bcm5301x.h>

#include <linux/mtd/mtd.h>
#include <linux/mtd/nand.h>
#include <linux/mtd/partitions.h>
#include <linux/romfs_fs.h>
#include <linux/cramfs_fs.h>
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,36)
#include <linux/squashfs_fs.h>
#else
/* #include <magic.h> */
#endif

#include <typedefs.h>
#include <osl.h>
#include <bcmutils.h>
#include <bcmnvram.h>
#include <bcmendian.h>
#include <hndsoc.h>
#include <siutils.h>
#include <hndcpu.h>
#include <hndpci.h>
#include <pcicfg.h>
#include <bcmdevs.h>
#include <trxhdr.h>
#ifdef HNDCTF
#include <ctf/hndctf.h>
#endif /* HNDCTF */
#include <hndsflash.h>
#ifdef CONFIG_MTD_NFLASH
#include <hndnand.h>
#endif
#include <linux/memblock.h>

extern char __initdata saved_root_name[];

/* Global SB handle */
si_t *bcm947xx_sih = NULL;
DEFINE_SPINLOCK(bcm947xx_sih_lock);
EXPORT_SYMBOL(bcm947xx_sih);
EXPORT_SYMBOL(bcm947xx_sih_lock);

/* Convenience */
#define sih bcm947xx_sih
#define sih_lock bcm947xx_sih_lock

#define WATCHDOG_MIN    3000    /* milliseconds */
extern int panic_timeout;
extern int panic_on_oops;

#ifdef HNDCTF
ctf_t *kcih = NULL;
EXPORT_SYMBOL(kcih);
ctf_attach_t ctf_attach_fn = NULL;
EXPORT_SYMBOL(ctf_attach_fn);
#endif /* HNDCTF */


struct dummy_super_block {
	u32	s_magic ;
};

/* This is the main reference clock 25MHz from external crystal */
static struct clk clk_ref = {
	.name = "Refclk",
	.rate = 25 * 1000000,	/* run-time override */
	.fixed = 1,
	.type	= CLK_XTAL,
};


static struct clk_lookup board_clk_lookups[] = {
	{
	.con_id		= "refclk",
	.clk		= &clk_ref,
	}
};

extern int _memsize;

void __init board_map_io(void)
{
early_printk("board_map_io\n");
	/* Install clock sources into the lookup table */
	clkdev_add_table(board_clk_lookups, 
			ARRAY_SIZE(board_clk_lookups));

	/* Map SoC specific I/O */
	soc_map_io( &clk_ref );
}

void __init board_init_early(void)
{
	init_dma_coherent_pool_size(SZ_1M * 2);
}

void __init board_init_irq(void)
{
early_printk("board_init_irq\n");
	soc_init_irq();
	
	/* serial_setup(sih); */
}

void __init board_init_timer(void)
{
early_printk("board_init_timer\n");
	soc_init_timer();
}

static int __init rootfs_mtdblock(void)
{
	int bootdev;
	int knldev;
	int block = 0;
#ifdef CONFIG_FAILSAFE_UPGRADE
	char *img_boot = nvram_get(BOOTPARTITION);
#endif

	bootdev = soc_boot_dev((void *)sih);
	knldev = soc_knl_dev((void *)sih);

	/* NANDBOOT */
	if (bootdev == SOC_BOOTDEV_NANDFLASH &&
	    knldev == SOC_KNLDEV_NANDFLASH) {
#ifdef CONFIG_FAILSAFE_UPGRADE
		if (img_boot && simple_strtol(img_boot, NULL, 10))
			return 5;
		else
			return 3;
#else
		return 3;
#endif
	}

	/* SFLASH/PFLASH only */
	if (bootdev != SOC_BOOTDEV_NANDFLASH &&
	    knldev != SOC_KNLDEV_NANDFLASH) {
#ifdef CONFIG_FAILSAFE_UPGRADE
		if (img_boot && simple_strtol(img_boot, NULL, 10))
			return 4;
		else
			return 2;
#else
		return 2;
#endif
	}

#ifdef BCMCONFMTD
	block++;
#endif
#ifdef CONFIG_FAILSAFE_UPGRADE
	if (img_boot && simple_strtol(img_boot, NULL, 10))
		block += 2;
#endif
	/* Boot from norflash and kernel in nandflash */
	return block+3;
}

static void __init brcm_setup(void)
{
	/* Get global SB handle */
	sih = si_kattach(SI_OSH);

#if 0
	if (strncmp(boot_command_line, "root=/dev/mtdblock", strlen("root=/dev/mtdblock")) == 0)
		sprintf(saved_root_name, "/dev/mtdblock%d", rootfs_mtdblock());
#endif

	/* Set panic timeout in seconds */
	panic_timeout = WATCHDOG_MIN / 1000;
	panic_on_oops = WATCHDOG_MIN / 1000;
}

static void __init brcm_identify_rev(void)
{
	uint32_t chipid;
	uint32_t chiprev;

	chipid = readl(SOC_CHIPCOMON_A_BASE_VA + 0 /* ChipcommonA_ChipID */);
	chiprev = ((chipid & 0x000F0000) >> 16);

	system_rev = chiprev;
}

void __init board_init(void)
{
early_printk("board_init\n");
	brcm_setup();
	brcm_identify_rev();
	/*
	 * Add common platform devices that do not have board dependent HW
	 * configurations
	 */
	soc_add_devices();

	return;
}

static void __init board_fixup(
	struct tag *t,
	char **cmdline)
{
	u32 mem_size, lo_size ;
        early_printk("board_fixup\n" );

#ifdef CONFIG_MACH_BRCM_NS_QT
	/* Fuxup reference clock rate */
	if (desc->nr == MACH_TYPE_BRCM_NS_QT )
		clk_ref.rate = 17594;   /* Emulator ref clock rate */
#endif

	mem_size = _memsize;

	early_printk("board_fixup: mem=%uMiB\n", mem_size >> 20);

	lo_size = min(mem_size, DRAM_MEMORY_REGION_SIZE);

	/* Cradlepoint:
	 * meminfo was taken out of kernel in
	 * 1c2f87c22566cd057bc8cde10c37ae9da1a1bb76
	 * */
	memblock_add(PHYS_OFFSET, lo_size);

	if (lo_size == mem_size)
		return;

	memblock_add(PHYS_OFFSET2, (mem_size - lo_size));
}

static void brcm_cpu_reset(enum reboot_mode mode, const char *cmd)
{
#ifdef CONFIG_OUTER_CACHE_SYNC
	outer_cache.sync = NULL;
#endif
#ifdef CONFIG_PLAT_BCM58522  /* WLENT_AP */
	soc_dmu_restart();
#else
	hnd_cpu_reset(sih);
#endif
}

#if (((IO_BASE_VA >> 18) & 0xfffc) != 0x3c40)
#error IO_BASE_VA 
#endif

MACHINE_START(BRCM_NS, "Northstar Prototype")
   .atag_offset = CONFIG_BOARD_PARAMS_PHYS,
   .smp	= smp_ops(brcm_smp_ops),
   .fixup = board_fixup,			/* Opt. early setup_arch() */
   .map_io = board_map_io,			/* Opt. from setup_arch() */
   .init_irq = board_init_irq,			/* main.c after setup_arch() */
   .init_early = board_init_early,
   .init_time = board_init_timer,		/* main.c after IRQs */
   .init_machine = board_init,			/* Late archinitcall */
#ifdef CONFIG_ZONE_DMA
   .dma_zone_size = SZ_128M,
#endif
   .restart = brcm_cpu_reset
MACHINE_END

#ifdef CONFIG_MACH_BRCM_NS_QT
MACHINE_START(BRCM_NS_QT, "Northstar Emulation Model")
   .atag_offset = CONFIG_BOARD_PARAMS_PHYS,
   .smp	= smp_ops(brcm_smp_ops),
   .fixup = board_fixup,			/* Opt. early setup_arch() */
   .map_io = board_map_io,			/* Opt. from setup_arch() */
   .init_irq = board_init_irq,			/* main.c after setup_arch() */
   .init_early = board_init_early,
   .init_time = board_init_timer,		/* main.c after IRQs */
   .init_machine = board_init,			/* Late archinitcall */
#ifdef CONFIG_ZONE_DMA
   .dma_zone_size = SZ_128M,
#endif
   .restart = brcm_cpu_reset
MACHINE_END
#endif

static spinlock_t *mtd_partition_lock = NULL;

spinlock_t *partitions_lock_init(void)
{
	if (!mtd_partition_lock) {
		mtd_partition_lock = (spinlock_t *)kzalloc(sizeof(spinlock_t), GFP_KERNEL);
		if (!mtd_partition_lock)
			return NULL;

		spin_lock_init( mtd_partition_lock );
	}
	return mtd_partition_lock;
}
EXPORT_SYMBOL(partitions_lock_init);

static struct nand_hw_control *nand_hwcontrol = NULL;
struct nand_hw_control *nand_hwcontrol_lock_init(void)
{
	if (!nand_hwcontrol) {
		nand_hwcontrol = (struct nand_hw_control *)kzalloc(sizeof(struct nand_hw_control), GFP_KERNEL);
		if (!nand_hwcontrol)
			return NULL;

		spin_lock_init(&nand_hwcontrol->lock);
		init_waitqueue_head(&nand_hwcontrol->wq);
	}
	return nand_hwcontrol;
}
EXPORT_SYMBOL(nand_hwcontrol_lock_init);

/* Find out prom size */
static uint32 boot_partition_size(uint32 flash_phys) {
	uint32 bootsz, *bisz;

	/* Default is 256K boot partition */
	bootsz = 256 * 1024;

	/* Do we have a self-describing binary image? */
	bisz = (uint32 *)(flash_phys + BISZ_OFFSET);
	if (bisz[BISZ_MAGIC_IDX] == BISZ_MAGIC) {
		int isz = bisz[BISZ_DATAEND_IDX] - bisz[BISZ_TXTST_IDX];

		if (isz > (1024 * 1024))
			bootsz = 2048 * 1024;
		else if (isz > (512 * 1024))
			bootsz = 1024 * 1024;
		else if (isz > (256 * 1024))
			bootsz = 512 * 1024;
		else if (isz <= (128 * 1024))
			bootsz = 128 * 1024;
	}
	return bootsz;
}

#if defined(BCMCONFMTD)
#define MTD_PARTS 1
#else
#define MTD_PARTS 0
#endif
#if defined(PLC)
#define PLC_PARTS 1
#else
#define PLC_PARTS 0
#endif
#if defined(CONFIG_FAILSAFE_UPGRADE)
#define FAILSAFE_PARTS 2
#else
#define FAILSAFE_PARTS 0
#endif
#if defined(CONFIG_CRASHLOG)
#define CRASHLOG_PARTS 1
#else
#define CRASHLOG_PARTS 0
#endif
/* boot;nvram;kernel;rootfs;empty */
#define FLASH_PARTS_NUM	(8+MTD_PARTS+PLC_PARTS+FAILSAFE_PARTS+CRASHLOG_PARTS)
#define FLASH_PARTS_NUM	(8+MTD_PARTS+PLC_PARTS+FAILSAFE_PARTS+CRASHLOG_PARTS)

static struct mtd_partition bcm947xx_flash_parts[FLASH_PARTS_NUM] = {{0}};

static uint lookup_flash_rootfs_offset(struct mtd_info *mtd, int *trx_off, size_t size,
				       uint32 *trx_size)
{
	struct romfs_super_block *romfsb;
	struct cramfs_super *cramfsb;
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,36)
	struct squashfs_super_block *squashfsb;
#else
	struct dummy_super_block *squashfsb;
#endif
	struct trx_header *trx;
	unsigned char buf[512];
	int off;
	size_t len;

	romfsb = (struct romfs_super_block *) buf;
	cramfsb = (struct cramfs_super *) buf;
	squashfsb = (void *) buf;
	trx = (struct trx_header *) buf;

	/* Look at every 64 KB boundary */
	for (off = *trx_off; off < size; off += (64 * 1024)) {
		memset(buf, 0xe5, sizeof(buf));

		/*
		 * Read block 0 to test for romfs and cramfs superblock
		 */
		if (mtd_read(mtd, off, sizeof(buf), &len, buf) ||
		    len != sizeof(buf))
			continue;

		/* Try looking at TRX header for rootfs offset */
		if (le32_to_cpu(trx->magic) == TRX_MAGIC) {
			// printk("trx off 0x%x\n", off);
			*trx_off = off;
			*trx_size = le32_to_cpu(trx->len);
			if (trx->offsets[1] == 0)
				continue;
			/*
			 * Read to test for romfs and cramfs superblock
			 */
			off += le32_to_cpu(trx->offsets[1]);
			memset(buf, 0xe5, sizeof(buf));
			if (mtd_read(mtd, off, sizeof(buf), &len, buf) || len != sizeof(buf))
				continue;
		}

		/* Try looking at U-Boot header for rootfs offset */
		if (le32_to_cpu(trx->magic) == UBOOT_MAGIC) {
			//printk("trx off 0x%x, size 0x%x\n", off, swab32(trx->flag_version));
			*trx_off = off;
			*trx_size = swab32(trx->flag_version) + 0x20000;
			*trx_size &= ~(0x20000 - 1);

			/*
			 * Read to test for romfs and cramfs superblock
			 */
			off += *trx_size;
			memset(buf, 0xe5, sizeof(buf));
			if (mtd_read(mtd, off, sizeof(buf), &len, buf) || len != sizeof(buf))
				continue;
		}

		/* romfs is at block zero too */
		if (romfsb->word0 == ROMSB_WORD0 &&
		    romfsb->word1 == ROMSB_WORD1) {
			printk(KERN_NOTICE
			       "%s: romfs filesystem found at block %d\n",
			       mtd->name, off / mtd->erasesize);
			break;
		}

		/* so is cramfs */
		if (cramfsb->magic == CRAMFS_MAGIC) {
			printk(KERN_NOTICE
			       "%s: cramfs filesystem found at block %d\n",
			       mtd->name, off / mtd->erasesize);
			break;
		}

		if (squashfsb->s_magic == SQUASHFS_MAGIC) {
			printk(KERN_NOTICE
			       "%s: squash filesystem found at block %d\n",
			       mtd->name, off / mtd->erasesize);
			break;
		}
	}

	return off;
}

struct mtd_partition *
init_mtd_partitions(hndsflash_t *sfl_info, struct mtd_info *mtd, size_t size)
{
	int bootdev = SOC_BOOTDEV_SFLASH;
	int knldev = SOC_KNLDEV_NORFLASH;
	int nparts = 0;
	uint32 offset = 0;
	uint rfs_off = 0;
	uint vmlz_off = 0, knl_size = 0;
	uint32 top = 0;
	uint32 bootsz;
	uint32 trx_size = 0;
#ifdef CONFIG_CRASHLOG
	char create_crash_partition = 0;
#endif
	int config_size = 0;
#ifdef CONFIG_FAILSAFE_UPGRADE
	char *img_boot = nvram_get(BOOTPARTITION);
	char *imag_1st_offset = nvram_get(IMAGE_FIRST_OFFSET);
	char *imag_2nd_offset = nvram_get(IMAGE_SECOND_OFFSET);
	unsigned int image_first_offset = 0;
	unsigned int image_second_offset = 0;
	char dual_image_on = 0;

	/* The image_1st_size and image_2nd_size are necessary if the Flash does not have any
	 * image
	 */
	dual_image_on = (img_boot != NULL && imag_1st_offset != NULL && imag_2nd_offset != NULL);

	if (dual_image_on) {
		image_first_offset = simple_strtol(imag_1st_offset, NULL, 10);
		image_second_offset = simple_strtol(imag_2nd_offset, NULL, 10);
		printk("The first offset=%x, 2nd offset=%x\n", image_first_offset,
			image_second_offset);

	}
#endif	/* CONFIG_FAILSAFE_UPGRADE */

	//bootdev = soc_boot_dev((void *)sih);
	//knldev = soc_knl_dev((void *)sih);

	if (bootdev == SOC_BOOTDEV_NANDFLASH) {
		/* Do not init MTD partitions on NOR flash when NAND boot */
		return NULL;	
	}

	config_size = 2 * mtd->erasesize;

	if (knldev != SOC_KNLDEV_NANDFLASH) {
		vmlz_off = 0;
		rfs_off = lookup_flash_rootfs_offset(mtd, &vmlz_off, size, &trx_size);

		//printk(KERN_INFO "rfs_off 0x%x, vmlz_off 0x%x, trx_size 0x%x\n", rfs_off, vmlz_off, trx_size);

		if (vmlz_off == 0) {
			// Couldn't find a bootloader, so the size is 1MB
			vmlz_off = 0x100000;
		}

#if defined (PRODUCT_AER1600)

		/* 
		   Since the original BRCM TRX code is almost unreadable, I thought I'd simplify 
		   a bit for the U-Boot partitions.
		*/

#define BOOTLOADER_SIZE 0xa0000

		bcm947xx_flash_parts[nparts].name = "Bootloader";
		bcm947xx_flash_parts[nparts].size = BOOTLOADER_SIZE;
		bcm947xx_flash_parts[nparts].offset = top;
		//bcm947xx_flash_parts[nparts].mask_flags = MTD_WRITEABLE; /* forces on read only */
		nparts++;

		bcm947xx_flash_parts[nparts].name = "Config";
		bcm947xx_flash_parts[nparts].size = mtd->erasesize;
		bcm947xx_flash_parts[nparts].offset = BOOTLOADER_SIZE;
		//bcm947xx_flash_parts[nparts].mask_flags = MTD_WRITEABLE; /* forces on read only */
		nparts++;

		bcm947xx_flash_parts[nparts].name = "Other";
		bcm947xx_flash_parts[nparts].size = mtd->erasesize * 5;
		bcm947xx_flash_parts[nparts].offset = BOOTLOADER_SIZE + mtd->erasesize;
		bcm947xx_flash_parts[nparts].mask_flags = MTD_WRITEABLE; /* forces on read only */
		nparts++;

		/* Setup kernel MTD partition */
		bcm947xx_flash_parts[nparts].name = "Kernel";
		bcm947xx_flash_parts[nparts].size = trx_size;
		bcm947xx_flash_parts[nparts].mask_flags = MTD_WRITEABLE; /* forces on read only */
		bcm947xx_flash_parts[nparts].offset = vmlz_off;
		knl_size = bcm947xx_flash_parts[nparts].size;
		offset = bcm947xx_flash_parts[nparts].offset + knl_size;
		nparts++; 

		/* Setup rootfs MTD partition */
		bcm947xx_flash_parts[nparts].name = "RootFS";
		bcm947xx_flash_parts[nparts].size = mtd->size - (rfs_off + (2 * config_size + ROUNDUP(nvram_space, mtd->erasesize)));
		bcm947xx_flash_parts[nparts].offset = rfs_off;
		bcm947xx_flash_parts[nparts].mask_flags = MTD_WRITEABLE; /* forces on read only */
		offset = bcm947xx_flash_parts[nparts].offset + bcm947xx_flash_parts[nparts].size;
		nparts++;
	
#else
		/* Size pmon */
		bcm947xx_flash_parts[nparts].name = "Bootloader";
		bcm947xx_flash_parts[nparts].size = vmlz_off;
		bcm947xx_flash_parts[nparts].offset = top;
		//bcm947xx_flash_parts[nparts].mask_flags = MTD_WRITEABLE; /* forces on read only */
		nparts++;

		/* Setup kernel MTD partition */
		bcm947xx_flash_parts[nparts].name = "Kernel";
#ifdef CONFIG_FAILSAFE_UPGRADE
		if (dual_image_on) {
			bcm947xx_flash_parts[nparts].size = image_second_offset-image_first_offset;
		} else {
			bcm947xx_flash_parts[nparts].size = mtd->size - vmlz_off - 2 * config_size;

			/* Reserve for NVRAM */
			bcm947xx_flash_parts[nparts].size -= ROUNDUP(nvram_space, mtd->erasesize);
#ifdef PLC
			/* Reserve for PLC */
			bcm947xx_flash_parts[nparts].size -= ROUNDUP(0x1000, mtd->erasesize);
#endif
#ifdef BCMCONFMTD
			bcm947xx_flash_parts[nparts].size -= (mtd->erasesize *4);
#endif
		}
#else

		bcm947xx_flash_parts[nparts].size = mtd->size - vmlz_off - 2 * config_size;
		
#ifdef PLC
		/* Reserve for PLC */
		bcm947xx_flash_parts[nparts].size -= ROUNDUP(0x1000, mtd->erasesize);
#endif
		/* Reserve for NVRAM */
		bcm947xx_flash_parts[nparts].size -= ROUNDUP(nvram_space, mtd->erasesize);

#ifdef BCMCONFMTD
		bcm947xx_flash_parts[nparts].size -= (mtd->erasesize *4);
#endif
#endif	/* CONFIG_FAILSAFE_UPGRADE */

#ifdef CONFIG_CRASHLOG
		if ((bcm947xx_flash_parts[nparts].size - trx_size) >=
		    ROUNDUP(0x4000, mtd->erasesize)) {
			bcm947xx_flash_parts[nparts].size -= ROUNDUP(0x4000, mtd->erasesize);
			create_crash_partition = 1;
		} else {
			create_crash_partition = 0;
		}
#endif

		bcm947xx_flash_parts[nparts].mask_flags = MTD_WRITEABLE; /* forces on read only */
		bcm947xx_flash_parts[nparts].offset = vmlz_off;
		knl_size = bcm947xx_flash_parts[nparts].size;
		offset = bcm947xx_flash_parts[nparts].offset + knl_size;
		nparts++; 

		/* Setup rootfs MTD partition */
		bcm947xx_flash_parts[nparts].name = "RootFS";
		bcm947xx_flash_parts[nparts].size = knl_size - (rfs_off - vmlz_off);
		bcm947xx_flash_parts[nparts].offset = rfs_off;
		bcm947xx_flash_parts[nparts].mask_flags = MTD_WRITEABLE; /* forces on read only */
		offset = bcm947xx_flash_parts[nparts].offset + bcm947xx_flash_parts[nparts].size;
		nparts++;
	
#endif     /* AER1600 */

#if defined(CONFIG_CRASHLOG) && defined(BCMDBG)
		if (create_crash_partition) {
			/* Setup crash MTD partition */
			bcm947xx_flash_parts[nparts].name = "crash";
			bcm947xx_flash_parts[nparts].size = ROUNDUP(0x4000, mtd->erasesize);
			bcm947xx_flash_parts[nparts].offset = offset;
			bcm947xx_flash_parts[nparts].mask_flags = 0;
			nparts++;
		}
#endif
#ifdef CONFIG_FAILSAFE_UPGRADE
		if (dual_image_on) {
			offset = image_second_offset;
			rfs_off = lookup_flash_rootfs_offset(mtd, &offset, size, &trx_size);
			vmlz_off = offset;
			/* Setup kernel2 MTD partition */
			bcm947xx_flash_parts[nparts].name = "linux2";
			bcm947xx_flash_parts[nparts].size = mtd->size - image_second_offset;
			/* Reserve for NVRAM */
			bcm947xx_flash_parts[nparts].size -= ROUNDUP(nvram_space, mtd->erasesize);

#ifdef BCMCONFMTD
			bcm947xx_flash_parts[nparts].size -= (mtd->erasesize *4);
#endif
#ifdef PLC
			/* Reserve for PLC */
			bcm947xx_flash_parts[nparts].size -= ROUNDUP(0x1000, mtd->erasesize);
#endif
			bcm947xx_flash_parts[nparts].offset = image_second_offset;
			knl_size = bcm947xx_flash_parts[nparts].size;
			offset = bcm947xx_flash_parts[nparts].offset + knl_size;
			nparts++;

			/* Setup rootfs MTD partition */
			bcm947xx_flash_parts[nparts].name = "rootfs2";
			bcm947xx_flash_parts[nparts].size =
				knl_size - (rfs_off - image_second_offset);
			bcm947xx_flash_parts[nparts].offset = rfs_off;
			/* forces on read only */
			bcm947xx_flash_parts[nparts].mask_flags = MTD_WRITEABLE;
			nparts++;
		}
#endif	/* CONFIG_FAILSAFE_UPGRADE */

	} else {
		bootsz = boot_partition_size(sfl_info->base);
		printk("Boot partition size = %d(0x%x)\n", bootsz, bootsz);
		/* Size pmon */
		bcm947xx_flash_parts[nparts].name = "boot";
		bcm947xx_flash_parts[nparts].size = bootsz;
		bcm947xx_flash_parts[nparts].offset = top;
		bcm947xx_flash_parts[nparts].mask_flags = MTD_WRITEABLE; /* forces on read only */
		offset = bcm947xx_flash_parts[nparts].size;
		nparts++;
	}

#ifdef BCMCONFMTD
	/* Setup CONF MTD partition */
	bcm947xx_flash_parts[nparts].name = "confmtd";
	bcm947xx_flash_parts[nparts].size = mtd->erasesize * 4;
	bcm947xx_flash_parts[nparts].offset = offset;
	offset = bcm947xx_flash_parts[nparts].offset + bcm947xx_flash_parts[nparts].size;
	nparts++;
#endif	/* BCMCONFMTD */

#ifdef PLC
	/* Setup plc MTD partition */
	bcm947xx_flash_parts[nparts].name = "plc";
	bcm947xx_flash_parts[nparts].size = ROUNDUP(0x1000, mtd->erasesize);
	bcm947xx_flash_parts[nparts].offset =
		size - (ROUNDUP(nvram_space, mtd->erasesize) + ROUNDUP(0x1000, mtd->erasesize));
	nparts++;
#endif

#ifdef TEST
	/* Setup 2nd Filemanager MTD partition */
	bcm947xx_flash_parts[nparts].name = "Test";
	bcm947xx_flash_parts[nparts].size = config_size * 2;
	bcm947xx_flash_parts[nparts].offset = size - 7 * mtd->erasesize;
	//bcm947xx_flash_parts[nparts].mask_flags = MTD_WRITEABLE; /* forces on read only */
	nparts++;
#endif

	/* Setup 2nd Filemanager MTD partition */
	bcm947xx_flash_parts[nparts].name = "2nd Filemanager";
	bcm947xx_flash_parts[nparts].size = config_size;
	bcm947xx_flash_parts[nparts].offset = size - (2 * config_size + ROUNDUP(nvram_space, mtd->erasesize));
	nparts++;

	/* Setup Filemanager MTD partition */
	bcm947xx_flash_parts[nparts].name = "Filemanager";
	bcm947xx_flash_parts[nparts].size = config_size;
	bcm947xx_flash_parts[nparts].offset = size - (config_size + ROUNDUP(nvram_space, mtd->erasesize));
	nparts++;

	/* Setup nvram MTD partition */
	bcm947xx_flash_parts[nparts].name = "nvram";
	bcm947xx_flash_parts[nparts].size = ROUNDUP(nvram_space, mtd->erasesize);
	bcm947xx_flash_parts[nparts].offset = size - bcm947xx_flash_parts[nparts].size;
	nparts++;

	return bcm947xx_flash_parts;
}

EXPORT_SYMBOL(init_mtd_partitions);

#ifdef	CONFIG_MTD_NFLASH

#define NFLASH_PARTS_NUM	7
static struct mtd_partition bcm947xx_nflash_parts[NFLASH_PARTS_NUM] = {{0}};

static uint
lookup_nflash_rootfs_offset(hndnand_t *nfl, struct mtd_info *mtd, int offset, size_t size)
{
	struct romfs_super_block *romfsb;
	struct cramfs_super *cramfsb;
	struct dummy_super_block *squashfsb;
	struct trx_header *trx;
	unsigned char *buf;
	uint blocksize, pagesize, mask, blk_offset, off, shift = 0;
	int ret;

	pagesize = nfl->pagesize;
	buf = (unsigned char *)kmalloc(pagesize, GFP_KERNEL);
	if (!buf) {
		printk("lookup_nflash_rootfs_offset: kmalloc fail\n");
		return 0;
	}

	romfsb = (struct romfs_super_block *) buf;
	cramfsb = (struct cramfs_super *) buf;
	squashfsb = (void *) buf;
	trx = (struct trx_header *) buf;

	/* Look at every block boundary till 16MB; higher space is reserved for application data. */
	blocksize = mtd->erasesize;
	for (off = offset; off < offset + size; off += blocksize) {
		mask = blocksize - 1;
		blk_offset = off & ~mask;
		if (hndnand_checkbadb(nfl, blk_offset) != 0)
			continue;
		memset(buf, 0xe5, pagesize);
		if ((ret = hndnand_read(nfl, off, pagesize, buf)) != pagesize) {
			printk(KERN_NOTICE
			       "%s: nflash_read return %d\n", mtd->name, ret);
			continue;
		}

		/* Try looking at TRX header for rootfs offset */
		if (le32_to_cpu(trx->magic) == TRX_MAGIC) {
			mask = pagesize - 1;
			off = offset + (le32_to_cpu(trx->offsets[1]) & ~mask) - blocksize;
			shift = (le32_to_cpu(trx->offsets[1]) & mask);
			romfsb = (struct romfs_super_block *)((unsigned char *)romfsb + shift);
			cramfsb = (struct cramfs_super *)((unsigned char *)cramfsb + shift);
			squashfsb = (struct squashfs_super_block *)
				((unsigned char *)squashfsb + shift);
			continue;
		}

		/* romfs is at block zero too */
		if (romfsb->word0 == ROMSB_WORD0 &&
		    romfsb->word1 == ROMSB_WORD1) {
			printk(KERN_NOTICE
			       "%s: romfs filesystem found at block %d\n",
			       mtd->name, off / blocksize);
			break;
		}

		/* so is cramfs */
		if (cramfsb->magic == CRAMFS_MAGIC) {
			printk(KERN_NOTICE
			       "%s: cramfs filesystem found at block %d\n",
			       mtd->name, off / blocksize);
			break;
		}

		if (squashfsb->s_magic == SQUASHFS_MAGIC) {
			printk(KERN_NOTICE
			       "%s: squash filesystem with lzma found at block %d\n",
			       mtd->name, off / blocksize);
			break;
		}

	}

	if (buf)
		kfree(buf);

	return shift + off;
}

struct mtd_partition *
init_nflash_mtd_partitions(hndnand_t *nfl, struct mtd_info *mtd, size_t size)
{
	int bootdev;
	int knldev;
	int nparts = 0;
	uint32 offset = 0;
	uint shift = 0;
	uint32 top = 0;
	uint32 bootsz;
#ifdef CONFIG_FAILSAFE_UPGRADE
	char *img_boot = nvram_get(BOOTPARTITION);
	char *imag_1st_offset = nvram_get(IMAGE_FIRST_OFFSET);
	char *imag_2nd_offset = nvram_get(IMAGE_SECOND_OFFSET);
	unsigned int image_first_offset = 0;
	unsigned int image_second_offset = 0;
	char dual_image_on = 0;

	/* The image_1st_size and image_2nd_size are necessary if the Flash does not have any
	 * image
	 */
	dual_image_on = (img_boot != NULL && imag_1st_offset != NULL && imag_2nd_offset != NULL);

	if (dual_image_on) {
		image_first_offset = simple_strtol(imag_1st_offset, NULL, 10);
		image_second_offset = simple_strtol(imag_2nd_offset, NULL, 10);
		printk("The first offset=%x, 2nd offset=%x\n", image_first_offset,
			image_second_offset);

	}
#endif	/* CONFIG_FAILSAFE_UPGRADE */

	bootdev = soc_boot_dev((void *)sih);
	knldev = soc_knl_dev((void *)sih);

	if (bootdev == SOC_BOOTDEV_NANDFLASH) {
		bootsz = boot_partition_size(nfl->base);
		if (bootsz > mtd->erasesize) {
			/* Prepare double space in case of bad blocks */
			bootsz = (bootsz << 1);
		} else {
			/* CFE occupies at least one block */
			bootsz = mtd->erasesize;
		}
		printk("Boot partition size = %d(0x%x)\n", bootsz, bootsz);

		/* Size pmon */
		bcm947xx_nflash_parts[nparts].name = "boot";
		bcm947xx_nflash_parts[nparts].size = bootsz;
		bcm947xx_nflash_parts[nparts].offset = top;
		bcm947xx_nflash_parts[nparts].mask_flags = MTD_WRITEABLE; /* forces on read only */
		offset = bcm947xx_nflash_parts[nparts].size;
		nparts++;

		/* Setup NVRAM MTD partition */
		bcm947xx_nflash_parts[nparts].name = "nvram";
		bcm947xx_nflash_parts[nparts].size = NFL_BOOT_SIZE - offset;
		bcm947xx_nflash_parts[nparts].offset = offset;

		offset = NFL_BOOT_SIZE;
		nparts++;
	}

	if (knldev == SOC_KNLDEV_NANDFLASH) {
		/* Setup kernel MTD partition */
		bcm947xx_nflash_parts[nparts].name = "linux";
#ifdef CONFIG_FAILSAFE_UPGRADE
		if (dual_image_on) {
			bcm947xx_nflash_parts[nparts].size =
				image_second_offset - image_first_offset;
		} else
#endif
		{
			bcm947xx_nflash_parts[nparts].size =
				nparts ? (NFL_BOOT_OS_SIZE - NFL_BOOT_SIZE) : NFL_BOOT_OS_SIZE;
		}
		bcm947xx_nflash_parts[nparts].offset = offset;

		shift = lookup_nflash_rootfs_offset(nfl, mtd, offset,
			bcm947xx_nflash_parts[nparts].size);

#ifdef CONFIG_FAILSAFE_UPGRADE
		if (dual_image_on)
			offset = image_second_offset;
		else
#endif
		offset = NFL_BOOT_OS_SIZE;
		nparts++;

		/* Setup rootfs MTD partition */
		bcm947xx_nflash_parts[nparts].name = "rootfs";
#ifdef CONFIG_FAILSAFE_UPGRADE
		if (dual_image_on)
			bcm947xx_nflash_parts[nparts].size = image_second_offset - shift;
		else
#endif
		bcm947xx_nflash_parts[nparts].size = NFL_BOOT_OS_SIZE - shift;
		bcm947xx_nflash_parts[nparts].offset = shift;
		bcm947xx_nflash_parts[nparts].mask_flags = MTD_WRITEABLE;
		offset = NFL_BOOT_OS_SIZE;

		nparts++;

#ifdef CONFIG_FAILSAFE_UPGRADE
		/* Setup 2nd kernel MTD partition */
		if (dual_image_on) {
			bcm947xx_nflash_parts[nparts].name = "linux2";
			bcm947xx_nflash_parts[nparts].size = NFL_BOOT_OS_SIZE - image_second_offset;
			bcm947xx_nflash_parts[nparts].offset = image_second_offset;
			shift = lookup_nflash_rootfs_offset(nfl, mtd, image_second_offset,
			                                    bcm947xx_nflash_parts[nparts].size);
			nparts++;
			/* Setup rootfs MTD partition */
			bcm947xx_nflash_parts[nparts].name = "rootfs2";
			bcm947xx_nflash_parts[nparts].size = NFL_BOOT_OS_SIZE - shift;
			bcm947xx_nflash_parts[nparts].offset = shift;
			bcm947xx_nflash_parts[nparts].mask_flags = MTD_WRITEABLE;
			nparts++;
		}
#endif	/* CONFIG_FAILSAFE_UPGRADE */

	}

#ifdef PLAT_NAND_JFFS2
	/* Setup the remainder of NAND Flash as FFS partition */
	if( size > offset ) {
		bcm947xx_nflash_parts[nparts].name = "ffs";
		bcm947xx_nflash_parts[nparts].size = size - offset ;
		bcm947xx_nflash_parts[nparts].offset = offset;
		bcm947xx_nflash_parts[nparts].mask_flags = 0;
		bcm947xx_nflash_parts[nparts].ecclayout = mtd->ecclayout;
		nparts++;
	}
#endif

	return bcm947xx_nflash_parts;
}

/* LR: Calling this function directly violates Linux API conventions */
EXPORT_SYMBOL(init_nflash_mtd_partitions);
#else

/* Note: NFL_BOOT_OS_SIZE is now set to 32M for FAILSAFE issue */
#ifndef	NFL_BOOT_OS_SIZE
#define	NFL_BOOT_OS_SIZE	SZ_32M
#endif

/* LR: Here is how partition managers are intended to be */
static int parse_cfenand_partitions(struct mtd_info *mtd,
                             struct mtd_partition **pparts,
                             struct mtd_part_parser_data *pdata)
{
	unsigned size, offset, nparts ;
	static struct mtd_partition cfenand_parts[2] = {{0}};

	if ( pparts == NULL )
		return -EINVAL;

	/*
	NOTE:
	Can not call init_nflash_mtd_partitions)_ because it depends
	on the "other" conflicting driver for its operation,
	so instead this will just create two partitions
	so that jffs2 does not overwrite bootable areas.
	*/

	size = mtd->size ;
	nparts = 0;
	offset = pdata->origin ;

	printk("%s: slicing up %uMiB provisionally\n", __func__, size >> 20 );

	cfenand_parts[nparts].name = "cfenand";
	cfenand_parts[nparts].size = NFL_BOOT_OS_SIZE ;
	cfenand_parts[nparts].offset = offset ;
	cfenand_parts[nparts].mask_flags = MTD_WRITEABLE;
	cfenand_parts[nparts].ecclayout = mtd->ecclayout;
	nparts++;
	offset += NFL_BOOT_OS_SIZE ;

	cfenand_parts[nparts].name = "jffs2";
	cfenand_parts[nparts].size = size - offset ;
	cfenand_parts[nparts].offset = offset;
	cfenand_parts[nparts].mask_flags = 0;
	cfenand_parts[nparts].ecclayout = mtd->ecclayout;
	nparts++;

	*pparts = cfenand_parts;
	return nparts;
}

static struct mtd_part_parser cfenand_parser = {
        .owner = THIS_MODULE,
        .parse_fn = parse_cfenand_partitions,
        .name = "cfenandpart",
};

static int __init cfenenad_parser_init(void)
{
        return register_mtd_parser(&cfenand_parser);
}

module_init(cfenenad_parser_init);

#endif /* CONFIG_MTD_NFLASH */

#ifdef CONFIG_CRASHLOG
extern char *get_logbuf(void);
extern char *get_logsize(void);

void nvram_store_crash(void)
{
	struct mtd_info *mtd = NULL;
	int i;
	char *buffer;
	unsigned char buf[16];
	int buf_len;
	int len;

	printk("Trying to store crash\n");

	mtd = get_mtd_device_nm("crash");

	if (!IS_ERR(mtd)) {

		buf_len = get_logsize();
		buffer = get_logbuf();
		if (buf_len > mtd->size)
			buf_len = mtd->size;

		memset(buf,0,sizeof(buf));
		mtd->read(mtd, 0, sizeof(buf), &len, buf);
		for (len=0;len<sizeof(buf);len++)
			if (buf[len]!=0xff) {
				printk("Could not save crash, partition not clean\n");
				break;
			}
		if (len == sizeof(buf)) {
			mtd->write(mtd, 0, buf_len, &len, buffer);
			if (buf_len == len)
				printk("Crash Saved\n");
		}
	} else {
		printk("Could not find NVRAM partition\n");
	}
}
#endif /* CONFIG_CRASHLOG */
