/*
 * Platform memory layout definitions
 *
 * Note: due to dependencies in common architecture code
 * some mappings go in other header files.
 */
#ifndef __ASM_ARCH_MEMORY_H
#define __ASM_ARCH_MEMORY_H

/*
 * Main memory base address and size
 * are defined from the board-level configuration file
 */

#define PHYS_OFFSET		PLAT_PHYS_OFFSET

/* 2nd physical memory window */
#ifdef CONFIG_PLAT_BCM58522  /* WLENT_AP */
#define PHYS_OFFSET2		0x68000000
#else
#define PHYS_OFFSET2		0x98000000
#endif

/* We currently never run in ACP mode (for consumer SDK) */
#define ACP_WAR_ENAB()		0
#define ACP_WIN_LIMIT		0
#define arch_is_coherent()	0

#ifdef CONFIG_SPARSEMEM

#define MAX_PHYSMEM_BITS	32
#define SECTION_SIZE_BITS	27

/* bank page offsets */
#define PAGE_OFFSET1	(PAGE_OFFSET + SZ_128M)

#define __phys_to_virt(phys)						\
	((phys) >= PHYS_OFFSET2 ? (phys) - PHYS_OFFSET2 + PAGE_OFFSET1 :	\
	 (phys) - PHYS_OFFSET + PAGE_OFFSET)

#define __virt_to_phys(virt)						\
	 ((virt) >= PAGE_OFFSET1 ? (virt) - PAGE_OFFSET1 + PHYS_OFFSET2 :	\
	  (virt) - PAGE_OFFSET + PHYS_OFFSET)

#endif /* CONFIG_SPARSEMEM */

#ifdef CONFIG_PLAT_BCM58522  /* WLENT_AP - CRYPTO_OFFLOAD */
/* BCM5301x Reference Guide (Section 3) defines three regions of IO  memory, 
 * CORE, IDM, and ARMCORE. The CORE and IDM regions are contiguous, so they 
 * are combined into a single region for mapping and translation purposes
 */

#define IO_CORE_IDM_PA		0x18000000
#define IO_CORE_IDM_SIZE	  0x200000
#define IO_ARMCORE_PA		0x19000000
#define IO_ARMCORE_SIZE		  0x100000

#define IO_CORE_IDM_VA		VMALLOC_END
#define IO_ARMCORE_VA		(IO_CORE_IDM_VA + IO_CORE_IDM_SIZE)

#define IO_CORE_IDM_PV_DELTA	(IO_CORE_IDM_VA - IO_CORE_IDM_PA)
#define IO_ARMCORE_PV_DELTA	(IO_ARMCORE_VA  - IO_ARMCORE_PA)

#define HW_IO_VIRT_TO_PHYS(virt)			\
	(((virt) < IO_ARMCORE_VA) ?			\
	 ((virt) - IO_CORE_IDM_PV_DELTA) :		\
	 ((virt) - IO_ARMCORE_PV_DELTA))

#endif /* CONFIG_PLAT_BCM58522   WLENT_AP - CRYPTO_OFFLOAD */

#endif
