/*
* ARM A9 MPCORE Platform base
*/


#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/init.h>
#include <linux/io.h>
#include <linux/clk.h>
#include <linux/errno.h>
#include <linux/mm.h>
#include <linux/vmalloc.h>
#include <linux/smp.h>
#include <linux/clockchips.h>
#include <linux/arm_global_timer.h>
#include <linux/ioport.h>
#include <linux/cpumask.h>
#include <linux/irqchip/arm-gic.h>

#include <asm/mach/map.h>
#include <asm/smp_twd.h>

#include <plat/mpcore.h>
#include <mach/io_map.h>

/* Globals */
static void __iomem * periphbase;

void __iomem * scu_base_addr(void)
{
	return (periphbase + MPCORE_SCU_OFF);
}

static DEFINE_TWD_LOCAL_TIMER(twd_local_timer,
			      SOC_SCU_BASE_PA + MPCORE_LTIMER_OFF,
			      MPCORE_IRQ_LOCALTIMER);

static DEFINE_ARM_GLOBAL_TIMER(arm_global_timer,
			      SOC_SCU_BASE_PA + MPCORE_GTIMER_OFF,
			      MPCORE_IRQ_GLOBALTIMER);

void __init mpcore_map_io(void)
{
	struct map_desc desc;
	phys_addr_t base_addr;

	/* 
	 * Cortex A9 Architecture Manual specifies this as a way to get
	 * MPCORE PERHIPHBASE address at run-time
	 */
	asm( "mrc p15,4,%0,c15,c0,0 @ Read Configuration Base Address Register" 
		: "=&r" (base_addr) : : "cc" );

	printk(KERN_INFO "MPCORE found at 0x%lx\n", (long)base_addr); 

	/* Make sure SCU is where we expect */
	BUG_ON(base_addr != SOC_SCU_BASE_PA);

	/* Fix-map the entire PERIPHBASE 2*4K register block */
	desc.virtual = MPCORE_BASE_VA;
	desc.pfn = __phys_to_pfn(base_addr);
	desc.length = SZ_8K;
	desc.type = MT_DEVICE;

	iotable_init(&desc, 1);

	periphbase = (void *)MPCORE_BASE_VA;
}

void __init mpcore_init_gic(void)
{
	printk(KERN_INFO "MPCORE GIC init\n");

	/* Init GIC interrupt distributor and CPU interface */
	gic_init(0, 1, periphbase + MPCORE_GIC_DIST_OFF, periphbase + MPCORE_GIC_CPUIF_OFF);
}

void __init mpcore_init_timer(unsigned long perphclk_freq)
{
	int err;

	/* Init Local Timer */
	printk(KERN_INFO "MPCORE Private timer setup CPU%u\n", smp_processor_id());

	err = twd_local_timer_register(&twd_local_timer);
	if (err)
		pr_err("twd_local_timer_register failed %d\n", err);

	/* Init Global Timer */
	printk(KERN_INFO "MPCORE Global timer setup CPU%u\n", smp_processor_id());
	printk(KERN_INFO "MPCORE Global timer clock %luHz\n", perphclk_freq);

	err = arm_global_timer_register(&arm_global_timer, perphclk_freq);
	if (err)
		pr_err("arm_global_timer_register failed %d\n", err);
}
