/*
 * FILE NAME cpboard.h
 *
 * BRIEF MODULE DESCRIPTION
 *  Shared defines for different board configurations.
 *
 *  Author: CradlePoint Technology, Inc.  <source@cradlepoint.com>
 *          Steve Johnson <stevej@cradlepoint.com>
 *
 * Copyright 2012, CradlePoint Technology, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2
 *  as published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to:
 *  Free Software Foundation
 *  51 Franklin Street, Fifth Floor
 *  Boston, MA  02111-1301  USA
 */

#ifndef _CPBOARD_H_
#define _CPBOARD_H_

#define SERIAL_CONSOLE 1
#define NO_SECURITY    2
extern int rt3xxx_check_memo_reg(int key);

// Generic PIO and SYSCTRL addresses
/*
  Some of these defines exist in asm/rt2880/rt_mmap.h, but their
  base address is inconsistent with an ioremap call.
*/

/*
 * Address of RALINK_ Registers
 */
#define RALINK_SYSCTL_ADDR		0x10000000	// system control
#define RALINK_REG_GPIOMODE		0x60

#define RALINK_PRGIO_BASE		RALINK_SYSCTL_ADDR + 0x600 // Programmable I/O
#define RALINK_PRGIO_ADDR		0x600 // Programmable I/O -- Everything else will be an offset from here
#define RALINK_REG_PIOINT		(RALINK_PRGIO_ADDR + 0)
#define RALINK_REG_PIOEDGE		(RALINK_PRGIO_ADDR + 0x04)
#define RALINK_REG_PIORENA		(RALINK_PRGIO_ADDR + 0x08)
#define RALINK_REG_PIOFENA		(RALINK_PRGIO_ADDR + 0x0C)
#define RALINK_REG_PIODATA		(RALINK_PRGIO_ADDR + 0x20)
#define RALINK_REG_PIODIR		(RALINK_PRGIO_ADDR + 0x24)
#define RALINK_REG_PIOPOL		(RALINK_PRGIO_ADDR + 0x28)
#define RALINK_REG_PIOSET		(RALINK_PRGIO_ADDR + 0x2C)
#define RALINK_REG_PIORESET		(RALINK_PRGIO_ADDR + 0x30)
#define RALINK_REG_PIOTOGGLE		(RALINK_PRGIO_ADDR + 0x34)

#define RALINK_REG_PIO3924INT		(RALINK_PRGIO_ADDR + 0x38)
#define RALINK_REG_PIO3924EDGE		(RALINK_PRGIO_ADDR + 0x3C)
#define RALINK_REG_PIO3924RENA		(RALINK_PRGIO_ADDR + 0x40)
#define RALINK_REG_PIO3924FENA		(RALINK_PRGIO_ADDR + 0x44)
#define RALINK_REG_PIO3924DATA		(RALINK_PRGIO_ADDR + 0x48)
#define RALINK_REG_PIO3924DIR		(RALINK_PRGIO_ADDR + 0x4C)
#define RALINK_REG_PIO3924POL		(RALINK_PRGIO_ADDR + 0x50)
#define RALINK_REG_PIO3924SET		(RALINK_PRGIO_ADDR + 0x54)
#define RALINK_REG_PIO3924RESET		(RALINK_PRGIO_ADDR + 0x58)
#define RALINK_REG_PIO3924TOGGLE	(RALINK_PRGIO_ADDR + 0x5C)

#define RALINK_REG_PIO7140INT		(RALINK_PRGIO_ADDR + 0x60)
#define RALINK_REG_PIO7140EDGE		(RALINK_PRGIO_ADDR + 0x64)
#define RALINK_REG_PIO7140RENA		(RALINK_PRGIO_ADDR + 0x68)
#define RALINK_REG_PIO7140FENA		(RALINK_PRGIO_ADDR + 0x6C)
#define RALINK_REG_PIO7140DATA		(RALINK_PRGIO_ADDR + 0x70)
#define RALINK_REG_PIO7140DIR		(RALINK_PRGIO_ADDR + 0x74)
#define RALINK_REG_PIO7140POL		(RALINK_PRGIO_ADDR + 0x78)
#define RALINK_REG_PIO7140SET		(RALINK_PRGIO_ADDR + 0x7C)
#define RALINK_REG_PIO7140RESET		(RALINK_PRGIO_ADDR + 0x80)
#define RALINK_REG_PIO7140TOGGLE	(RALINK_PRGIO_ADDR + 0x84)

#define RALINK_REG_PIO9572INT		(RALINK_PRGIO_ADDR + 0x88)
#define RALINK_REG_PIO9572EDGE		(RALINK_PRGIO_ADDR + 0x8C)
#define RALINK_REG_PIO9572RENA		(RALINK_PRGIO_ADDR + 0x90)
#define RALINK_REG_PIO9572FENA		(RALINK_PRGIO_ADDR + 0x94)
#define RALINK_REG_PIO9572DATA		(RALINK_PRGIO_ADDR + 0x98)
#define RALINK_REG_PIO9572DIR		(RALINK_PRGIO_ADDR + 0x9C)
#define RALINK_REG_PIO9572POL		(RALINK_PRGIO_ADDR + 0xA0)
#define RALINK_REG_PIO9572SET		(RALINK_PRGIO_ADDR + 0xA4)
#define RALINK_REG_PIO9572RESET		(RALINK_PRGIO_ADDR + 0xA8)
#define RALINK_REG_PIO9572TOGGLE	(RALINK_PRGIO_ADDR + 0xAC)

// Utility functions
#define WRITE_PIO(val, addr) \
	writel(cpu_to_le32(val), base_addr + addr)
#define READ_PIO(addr) \
	le32_to_cpu(readl(base_addr + addr))

#if defined(CONFIG_RALINK_RT3052)
#define RALINK_SYSCTL_MEMO0        0x68
#define RALINK_SYSCTL_MEMO1        0x6C
#else
// Others don't have memo regs, use teststat instead
#define RALINK_SYSCTL_MEMO0        0x18
#define RALINK_SYSCTL_MEMO1        0x1C
#endif

// Kernel Events
//  These are platform-neutral
#define WPS_BUTTON_EVT 1
#define RESET_BUTTON_EVT 2
#define POWER_BUTTON_EVT 3
#define IN_5V_EVT 4
//#define PWR_FLAG_3G_EVT 5
#define DOCK_SENSE_EVT 6
#define LAN_WAN_SW_EVT 7
#define WIFI_ENABLE_EVT 8
#define SS_BUTTON_EVT 9
#define CURRENT_LIMIT_EVT 10
#define EXCARD_ATTACH_EVT 11
#define CONNECTOR_INPUT_EVT 12

#if defined(CONFIG_RALINK_RT3883)
#define GPIO_PINS 96
#else
#if defined(CONFIG_RALINK_RT3352)
#define GPIO_PINS 45
#else
#if defined(CONFIG_RALINK_MT7620)
#define GPIO_PINS 73
#else
#define GPIO_PINS 52
#endif
#endif
#endif

#endif
