/*
 * FILE NAME board_setup.h
 *
 * BRIEF MODULE DESCRIPTION
 *  Simple setup for different board configurations.
 *
 *  Author: CradlePoint Technology, Inc.  <source@cradlepoint.com>
 *          Steve Johnson <stevej@cradlepoint.com>
 *
 * Copyright 2012, CradlePoint Technology, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2
 *  as published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to:
 *  Free Software Foundation
 *  51 Franklin Street, Fifth Floor
 *  Boston, MA  02111-1301  USA
 */

#ifndef __BOARD_SETUP_H__
#define __BOARD_SETUP_H__

#if defined(PRODUCT_RUBY)

#define GPIO_PIN_MASK                   0x3e7f81
#define GPIO_PIN_MASK_24_39             0x0000 
#define GPIO_PIN_MASK_40_71             0xfff00000
#define GPIO_PIN_MASK_72_95             0x1

#define GPIO_OUTPUT_PIN_MASK            0x3c7001
#define GPIO_OUTPUT_PIN_MASK_24_39      0x0000
#define GPIO_OUTPUT_PIN_MASK_40_71      0x30e00000
#define GPIO_OUTPUT_PIN_MASK_72_95      0x1

// rising edge interrupts
#define GPIO_INT_PIN_MASK               0x020f80
#define GPIO_INT_PIN_MASK_24_39         0x00000
#define GPIO_INT_PIN_MASK_40_71         0x8C000000
#define GPIO_INT_PIN_MASK_72_95         0x1

// falling edge interrupts
#define GPIO_INT_FEDGE_PIN_MASK         0x020f80
#define GPIO_INT_FEDGE_PIN_MASK_24_39   0x00000
#define GPIO_INT_FEDGE_PIN_MASK_40_71   0x8C000000
#define GPIO_INT_FEDGE_PIN_MASK_72_95   0x1

#define GPIO_POL_MASK                   0x0
#define GPIO_POL_MASK_24_39             0x0
#define GPIO_POL_MASK_40_71             0x80000000
#define GPIO_POL_MASK_72_95             0x0

int pins_to_reset[] = {18, 19, 61, 65};
int pins_to_set[] = {0, 12, 13, 14, 20, 21, 62, 69, 72}; 
int leds_to_twiddle[] = {0, 12, 13, 14};

#define DELAY_TIME 3 * HZ

#endif //RUBY

#if defined(PRODUCT_CONGO)
#define GPIO_PIN_MASK                  0xfe7f81
#define GPIO_PIN_MASK_24_39            0x27ff
#define GPIO_PIN_MASK_40_71            0xfff00000
#define GPIO_PIN_MASK_72_95            0x1

#define GPIO_OUTPUT_PIN_MASK           0xfc5981
#define GPIO_OUTPUT_PIN_MASK_24_39     0x27ff
#define GPIO_OUTPUT_PIN_MASK_40_71     0x51f00000
#define GPIO_OUTPUT_PIN_MASK_72_95     0x1

#define GPIO_INT_PIN_MASK              0x002600    // rising edge interrupts
#define GPIO_INT_PIN_MASK_24_39        0x0000     // rising edge interrupts
#define GPIO_INT_PIN_MASK_40_71        0x8c000000 // rising edge interrupts
#define GPIO_INT_PIN_MASK_72_95        0

#define GPIO_INT_FEDGE_PIN_MASK         0x022600    // Falling edge.  Thermal IC
#define GPIO_INT_FEDGE_PIN_MASK_24_39   0x0000     //
#define GPIO_INT_FEDGE_PIN_MASK_40_71   0x8c000000  //
#define GPIO_INT_FEDGE_PIN_MASK_72_95   0

#define GPIO_POL_MASK                   0x000000    //
#define GPIO_POL_MASK_24_39             0x0000
#define GPIO_POL_MASK_40_71             0x80000000
#define GPIO_POL_MASK_72_95             0

int pins_to_reset[] = {18, 19, 22, 35, 61, 68};
int pins_to_set[] = {20, 21, 29, 30, 31, 32, 70, 72};
int leds_to_twiddle[] = {29, 30, 31, 32};
#define DELAY_TIME 6 * HZ

#endif /*Congo */

#if defined(PRODUCT_IBR200)

#define GPIO_PIN_MASK					0xc80681
#define GPIO_PIN_MASK_24_39				0x17ff
#define GPIO_PIN_MASK_40_71				0xfff00000
#define GPIO_PIN_MASK_72_95				0x0

#define GPIO_OUTPUT_PIN_MASK			0xc80080
#define GPIO_OUTPUT_PIN_MASK_24_39		0x11ff
#define GPIO_OUTPUT_PIN_MASK_40_71		0x10f00000
#define GPIO_OUTPUT_PIN_MASK_72_95		0x0

#define GPIO_INT_PIN_MASK				0x0
#define GPIO_INT_PIN_MASK_24_39			0x0
#define GPIO_INT_PIN_MASK_40_71			0x86000000
#define GPIO_INT_PIN_MASK_72_95			0x0

#define GPIO_INT_FEDGE_PIN_MASK			0x200
#define GPIO_INT_FEDGE_PIN_MASK_24_39	0x0
#define GPIO_INT_FEDGE_PIN_MASK_40_71	0x8e000000
#define GPIO_INT_FEDGE_PIN_MASK_72_95	0x0

#define GPIO_POL_MASK					0x0
#define GPIO_POL_MASK_24_39				0x0
#define GPIO_POL_MASK_40_71				0x80000000
#define GPIO_POL_MASK_72_95				0x0
int pins_to_reset[] = {7, 19, 22, 23, 28, 29, 61, 62, 63, 68 };
int pins_to_set[] = {24, 25, 26, 27, 30, 31, 32, 36, 60 };
int leds_to_twiddle[] = {29, 30, 31, 32};
#define DELAY_TIME 7 * HZ

#endif /*IBR200 */

#if defined(PRODUCT_CBA550)

#define GPIO_PIN_MASK 0x400401
#define GPIO_PIN_MASK_24_39 0x7e9
#define GPIO_PIN_MASK_40_71 0xd3200000
#define GPIO_PIN_MASK_72_95 0x0
#define GPIO_OUTPUT_PIN_MASK 0x400000
#define GPIO_OUTPUT_PIN_MASK_24_39 0x1e9
#define GPIO_OUTPUT_PIN_MASK_40_71 0x10200000
#define GPIO_OUTPUT_PIN_MASK_72_95 0x0
#define GPIO_INT_PIN_MASK 0x0
#define GPIO_INT_PIN_MASK_24_39 0x0
#define GPIO_INT_PIN_MASK_40_71 0x82000000
#define GPIO_INT_PIN_MASK_72_95 0x0
#define GPIO_INT_FEDGE_PIN_MASK 0x0
#define GPIO_INT_FEDGE_PIN_MASK_24_39 0x0
#define GPIO_INT_FEDGE_PIN_MASK_40_71 0x82000000
#define GPIO_INT_FEDGE_PIN_MASK_72_95 0x0
#define GPIO_POL_MASK 0x0
#define GPIO_POL_MASK_24_39 0x0
#define GPIO_POL_MASK_40_71 0x80000000
#define GPIO_POL_MASK_72_95 0x0
int pins_to_reset[] = {22, 29, 61, 68};
int pins_to_set[] = {24, 27, 30, 31, 32};
int leds_to_twiddle[] = {29, 30, 31, 32};
#define DELAY_TIME 7 * HZ

#endif /* CBA550 */

// Flash mapping:

#define MTD_BOOT_PART_SIZE	    0x30000
#define	MTD_CONFIG_PART_SIZE	0x10000
#define	MTD_FACTORY_PART_SIZE	0x10000
#if defined (CONFIG_RT2880_FLASH_32M)
#define MTD_FILEMGR_PART_SIZE   0x40000
#define MTD_PART_SIZE           0x2000000
#else
#if defined (CONFIG_RT2880_FLASH_16M)
#define MTD_FILEMGR_PART_SIZE   0x20000
#define MTD_PART_SIZE           0x1000000
#else
#if defined (CONFIG_RT2880_FLASH_8M)
#define MTD_FILEMGR_PART_SIZE   0x20000
#define MTD_PART_SIZE           0x800000
#else
#error "No flash size configuration found"
#endif
#endif
#endif

#ifdef CONFIG_RT2880_ROOTFS_IN_FLASH
#ifdef CONFIG_ROOTFS_IN_FLASH_NO_PADDING
#define CONFIG_MTD_KERNEL_PART_SIZ 0
#endif
#define MTD_ROOTFS_PART_SIZE	MTD_PART_SIZE - (MTD_BOOT_PART_SIZE + MTD_CONFIG_PART_SIZE \
												 + MTD_FACTORY_PART_SIZE + CONFIG_MTD_KERNEL_PART_SIZ)
#define	MTD_KERN_PART_SIZE	CONFIG_MTD_KERNEL_PART_SIZ
#else
#define MTD_KERN_PART_SIZE	MTD_PART_SIZE - (MTD_BOOT_PART_SIZE + MTD_CONFIG_PART_SIZE \
											 + MTD_FACTORY_PART_SIZE)
#endif


#endif // __BOARD_SETUP_H__
